#!/usr/bin/env python
import argparse
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import itertools
import os
import pandas as pd
import seaborn as sns
sns.set_context("talk")
sns.set_style("whitegrid")
sns.set_palette(sns.color_palette("colorblind"))


def is_existing_dir(arg):
    if not os.path.isdir(arg):
        parser.error(
            'Given path {} does not point to an existing directory. Exiting.'.format(
                arg))
    return arg

parser = argparse.ArgumentParser(description='''
Produces bar plots with clustering performance for a range of clustering data
sets. This is supposed to be called after running run_clustering_docker.sh
''')
parser.add_argument('clustering_dir', type=is_existing_dir, nargs='+',
                    help='Directories containing clustering results to be '
                         'plotted. Each directory specified here is only '
                         'allowed to contain a single clustering result. '
                         'Each directory is expected to contain one directory '
                         'per data set clustered which each contains exactly '
                         'one sub-directory containing the clustering result.'
                         'Note that the input order of the data sets is '
                         'preserved in the output plot.')
parser.add_argument('--plot_label', type=str, nargs='+',
                    help='Labels for the clustering_dir to be used in the '
                         'output plots. Number of plot_label must match the '
                         'number of clustering_dir and plot_label must be '
                         'given in the same order as clustering_dir. '
                         'If no plot_label argument is given, '
                         'clustering_dir argument will be used instead.')
parser.add_argument('--plot_dir', type=str, default='cluster_plots',
                    help='Directory to write output plots to. Directory is '
                         'if necessary created. Default: cluster_plots/')
parser.add_argument('--clustering_metric', type=str,
                    default='AdjustedRandIndex', help='The clustering metric '
                                'to plot. Default: Adjusted Rand Index')
parser.add_argument('--line_plot', action='store_true',
                    help='If this is set, a line plot instead of a bar plot '
                         'is drawn.')
args = parser.parse_args()
input_dirs = args.clustering_dir
line_plot = args.line_plot
if args.plot_label:
    if len(args.plot_label) != len(input_dirs):
        msg = 'Number of clustering_dir ({:d}) does not match number of ' \
              'plot_label ({:d}). Exiting!'.format(len(input_dirs),
                                                    len(args.plot_label))
        raise ValueError(msg)
    input_dir_to_label = dict(zip(input_dirs, args.plot_label))
else:
    input_dir_to_label = dict(zip(input_dirs,
                                  [i.strip().strip(os.sep)
                                   for i in input_dirs]))
plot_dir = args.plot_dir
if not os.path.isdir(plot_dir):
    if os.path.isfile(plot_dir):
        msg = 'Given plot_dir {} is a file. Must be non-existing or a ' \
              'directory. Exiting!'.format(plot_dir)
        raise ValueError(msg)
    os.makedirs(plot_dir)
output_plot_basename = os.path.join(plot_dir, 'cluster_analysis')
output_tsv = os.path.join(plot_dir, 'cluster_analysis.tsv')

# map data set sub-directory file names to labels used in plots
data_set_to_label = {'cliques-high': 'Rfam-cliques High\n(Mean PSI 0.73)',
                     'cliques-low': 'Rfam-cliques Low\n(Mean PSI 0.50)',
                     'cliques-medium': 'Rfam-cliques Medium\n(Mean PSI 0.63)',
                     'rfamome': 'Rfam-ome\n(Mean PSI 0.78)'}
data_set_plot_order = ['cliques-low', 'cliques-medium', 'cliques-high', 'rfamome']

cluster_metric_to_label = {'Homogeneity': 'Homogeneity',
                            'Completeness': 'Completeness',
                            'V-measure': 'V-measure',
                            'RandIndex': 'Rand Index',
                            'AdjustedRandIndex': 'Adjusted Rand Index',
                            'Matthew\'s correlation coefficient': '"Matthew\'s correlation coefficient',
                            'ClusterCount': 'Cluster Count',
                            'TrueClassCount': 'True Class Count'}
clustering_metric = args.clustering_metric
if clustering_metric not in cluster_metric_to_label:
    msg = 'Did not recognize specified clustering metric {}'.format(
        clustering_metric)
    raise ValueError(msg)


def load_clustering_df(clustering_result_path, clustering_label, dataset_label):
    tmp_df = pd.read_csv(clustering_result_path)
    tmp_df['Clustering method'] = [clustering_label]
    tmp_df['Data set'] = [dataset_label]
    tmp_df.columns = [c.strip() for c in tmp_df.columns]
    return tmp_df

cluster_df = None
for i, input_dir_data_set in enumerate(
        itertools.product(input_dirs, data_set_plot_order)):
    input_dir, data_set = input_dir_data_set
    data_set_dir = os.path.join(input_dir, data_set)
    dir_contents = os.listdir(data_set_dir)
    if len(dir_contents) != 1:
        msg = 'Directory {} is expected to contain exactly one sub-directory ' \
              'but contains:'.format(data_set_dir, ', '.join(dir_contents))
        raise ValueError(msg)
    clustering_file = os.path.join(data_set_dir, dir_contents[0],
                                   'cluster_evaluation',
                                   'cluster_analysis.tsv')
    curr_df = load_clustering_df(clustering_file, input_dir_to_label[input_dir],
                                 data_set_to_label[data_set])
    if i == 0:
        cluster_df = curr_df
    else:
        cluster_df = pd.concat([cluster_df, curr_df])

cluster_df.to_csv(output_tsv, sep='\t')
if line_plot:
    plot_kind = 'point'
else:
    plot_kind = 'bar'
g = sns.factorplot(x='Data set', y=clustering_metric,
                   hue='Clustering method', data=cluster_df, kind=plot_kind,
                   legend=False, aspect=2, size=6)
# If we want to use add line plots, try something like:
# g = g.map_dataframe(sns.pointplot, x="Data set", y=" AdjustedRandIndex",
#                     hue="Clustering method", scale=.7)
(g.set_axis_labels('', cluster_metric_to_label[clustering_metric])
    .set(ylim=(0, 1)))
plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
           ncol=2, mode="expand", borderaxespad=0.)

output_formats = ['.eps', '.svg', '.png']
for format in output_formats:
    out_file = output_plot_basename + format
    g.savefig(out_file)