#!/usr/bin/perl

############################################################################
# This script runs PETfold on a fasta alignment
# Returns the consensus structure of the alignment with 
#   filterd base pairs based on their PETfold reliablities
# 
############################################################################

#use feature ':5.10';
use File::Basename;
use lib dirname($0);    # search skript directory for modules
use strict 'vars';
use warnings;
use Getopt::Long;
use Pod::Usage;
use List::Util qw/ min max /;



use Cwd qw(getcwd abs_path);
use File::Temp qw(tempdir);
use File::Copy;
use POSIX qw(ceil);

use FindBin;                 # locate this script
use lib "$FindBin::Bin/../";  # use the parent directory
use Config::AlignClustConfig;

use lib "$FindBin::Bin/../lib/";  # use the parent directory

use MultipleAlignment::alifold;
use StructureLibrary::Tools;

##script name
my $SCRIPT_NAME = basename($0);


=head1 NAME


ss ssss dfd
=head1 SYNOPSIS

Options:

  HELP
  -help   brief help message
  -man    full documentation

COMPULSORY
  -align-fasta <STRING> alignment file name
  -threshold      <FLOAT> e.g. 0.001
  			Covariance bonus threshold, report basepairs of consensus structure above threshold
  			 
  -debug  additional debug output
  -local-config   if a local configuration is to be used, path to the local configuration file


  DEFAULT VALUES
  -threshold   100
=head1 DESCRIPTION

=cut

###############################################################################
# sub declrations
###############################################################################
#sub parse_real_pp;

###############################################################################
# PARSE COMMAND LINE OPTIONS
###############################################################################

# command line options
my (
  $i_help,               $i_man,          $i_debug, $i_local_config,
  $i_struct_stk, $i_threshold, $i_consensus_in, $i_tmp_dir, $i_job_id,
  $i_tool
);


$i_threshold = 0.1;
my $options = GetOptions(
  "help"            => \$i_help,
  "man"             => \$i_man,
  "tool=s"           => \$i_tool,
  "debug"           => \$i_debug,
  "local-config=s" => \$i_local_config,
  "align-file=s"   => \$i_struct_stk,
  "working-dir=s"	=> \$i_tmp_dir,
  "consensus-in=s"	=> \$i_consensus_in,
  "threshold=f"     => \$i_threshold, 
  "job-id=s"		=> \$i_job_id
  );

pod2usage( -exitstatus => 1, -verbose => 1 ) if $i_help;
pod2usage( -exitstatus => 0, -verbose => 2 ) if $i_man;
($options) or pod2usage(2);

if ($i_local_config) {
  (-e $i_local_config) or pod2usage("error: local configuration file does not exist: $i_local_config\n");
}

# check compulsory options
($i_tmp_dir) or pod2usage("Error: no tmp directory has been defined\n");
(-e $i_tmp_dir) or pod2usage("Error: tmp directory does not exist!\n");
$i_tmp_dir = abs_path($i_tmp_dir);

($i_struct_stk) or pod2usage("Error: the option -align-stk is compulsory!\n");
#( -e $i_consensus_in ) or pod2usage("Error: consensus_in not found!: $i_consensus_in\n");
( -e $i_struct_stk ) or pod2usage("Error: no such file - $i_struct_stk!\n");
$i_struct_stk = abs_path($i_struct_stk);

if ($i_tool ne 'alifold' and $i_tool ne 'rscape') {
	pod2usage "wrong tool name: $i_tool:\n";
}

# check other options and set default values

# set the appropriate exports for old and new cluster respectively

###############################################################################
# Overwrite global CONFIG with local configuration
###############################################################################
my $CONFIG_REF;
if ($i_local_config) {
    $CONFIG_REF = Config::AlignClustConfig::load_config($i_local_config);
} else {
    $CONFIG_REF = Config::AlignClustConfig::load_config();
}
my %CONFIG = %{$CONFIG_REF};

###############################################################################
# GLOBAL VARIABLES
###############################################################################
#my $PETFOLD_BIN="/home/miladim/software/PETfold/bin/PETfold";  
#my $RNADIST_BIN="/home/miladim/software/bin/RNAdistance";  
my $alifold_loc = $CONFIG{ALIFOLD_BIN};
my $rscape_loc = $CONFIG{RSCAPE_BIN};

my $ALIOUT_FOLDER = "/alifold_output";
my $alifold_LOG = "ali_log";
my $rscape_LOG = "rscape_log";



###############################################################################
# end handler for temporary directory
# adds an error handler that deletes the directory in case of error
# SIGUSR{1/2} are sent by the sge prior to the uncatchable SIGKILL if the
# option -notify was set
###############################################################################
$SIG{'INT'}  = 'end_handler';
$SIG{'TERM'} = 'end_handler';
$SIG{'ABRT'} = 'end_handler';
$SIG{'USR1'} = 'end_handler';
$SIG{'USR2'} = 'end_handler';

sub end_handler {
  print STDERR "signal ", $_[0], " caught, cleaning up temporary files\n";

  # change into home directory. deletion of the temporary directory will
  # fail if it is the current working directory
  chdir();
  File::Temp::cleanup();
  die();
}

###############################################################################
# EXECUTION CODE
###############################################################################

# set up tmp directory

$ALIOUT_FOLDER .= "_$i_job_id" if ($i_job_id);

$ALIOUT_FOLDER = $i_tmp_dir . $ALIOUT_FOLDER;

##create out directory if necessary
system "mkdir $ALIOUT_FOLDER" if (! (-e $ALIOUT_FOLDER) );
$ALIOUT_FOLDER = abs_path($ALIOUT_FOLDER);
	my $max_coverage_ratio = 1.0;
	my $min_coverage_ratio = 0.2;

if ($i_tool eq 'alifold') {
	##call modified RNAalifold 
	my $params_alifold = "--input-format=S " ;
	my $call_alifold = "$alifold_loc/RNAalifold $params_alifold < $i_struct_stk > $ALIOUT_FOLDER/$alifold_LOG";
	print STDOUT "call_alifold: $call_alifold\n";
	system ("$call_alifold ");
	my $covarying_struct = MultipleAlignment::alifold::parse_alifold_outlog_get_consensus (
		"$ALIOUT_FOLDER/$alifold_LOG"); #, $i_consensus_in, $i_threshold);
	#print "i_consensus_in:   $i_consensus_in\n";
	print "covarying_struct: $covarying_struct\n";
}
elsif($i_tool eq 'rscape') {
	my $basename = (get_path_base_suffix($i_struct_stk))[1];
	
	##call Rscape 
	my $params_rscape = "--nofigures --gapthresh 1.0 -E $i_threshold --outdir $ALIOUT_FOLDER/ " ;
	my $call_rscape = "$rscape_loc/R-scape $params_rscape $i_struct_stk > $ALIOUT_FOLDER/$rscape_LOG";
	print STDOUT "call_rscape: $call_rscape\n";
	system ("$call_rscape ");
	
	my $covarying_struct = MultipleAlignment::alifold::parse_rscape_outlog_get_covarying_bps
		 ("$ALIOUT_FOLDER/$basename.sorted.out", $i_consensus_in, $i_threshold, $max_coverage_ratio, $min_coverage_ratio);
	print "i_consensus_in:   $i_consensus_in\n";
	print "covarying_struct: $covarying_struct\n";
}

